@@ -24,15 +24,17 @@
 
 #include "randr.h"
 
-class RandROutput;
+class OutputConfig;
 
 class OutputGraphicsItem : public QObject, public QGraphicsRectItem
 {
 	Q_OBJECT
 public:
-	OutputGraphicsItem(RandROutput *output);
+	OutputGraphicsItem(OutputConfig *config);
 	~OutputGraphicsItem();
 
+	void configUpdated(); // updates from OutputConfig
+
 	OutputGraphicsItem *left() const;
 	OutputGraphicsItem *right() const;
 	OutputGraphicsItem *top() const;
@@ -45,8 +47,6 @@ public:
 
 	bool isConnected();
     
-	void updateText();
-
 protected:
 	void disconnect();
 	virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
@@ -61,7 +61,7 @@ private:
 	OutputGraphicsItem *m_top;
 	OutputGraphicsItem *m_bottom;
 
-	RandROutput *m_output;
+	OutputConfig *m_config;
 	QGraphicsTextItem *m_text;
 
 
