@@ -26,7 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "scene.h"
 
 #include <QStack>
-#include <QMap>
+#include <QHash>
 
 
 class KService;
@@ -47,6 +47,8 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual void paintWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data );
         virtual void postPaintWindow( EffectWindow* w );
 
+        bool providesResizeEffect();
+
         virtual void drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data );
 
         virtual void buildQuads( EffectWindow* w, WindowQuadList& quadList );
@@ -61,6 +63,7 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual int currentDesktop() const;
         virtual int numberOfDesktops() const;
         virtual void setCurrentDesktop( int desktop );
+        virtual void setNumberOfDesktops( int desktops );
         virtual QSize desktopGridSize() const;
         virtual int desktopGridWidth() const;
         virtual int desktopGridHeight() const;
@@ -134,9 +137,13 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual unsigned long xrenderBufferPicture();
         virtual void reconfigure();
         virtual void registerPropertyType( long atom, bool reg );
+        virtual QByteArray readRootProperty( long atom, long type, int format ) const;
+        virtual void deleteRootProperty( long atom ) const;
 
         virtual bool hasDecorationShadows() const;
 
+        virtual bool decorationsHaveAlpha() const;
+
         // internal (used by kwin core or compositing code)
         void startPaint();
         void windowUserMovedResized( EffectWindow* c, bool first, bool last );
@@ -165,6 +172,7 @@ class EffectsHandlerImpl : public EffectsHandler
         void grabbedKeyboardEvent( QKeyEvent* e );
         bool hasKeyboardGrab() const;
         void propertyNotify( EffectWindow* c, long atom );
+        void numberDesktopsChanged( int old );
 
         bool loadEffect( const QString& name );
         void toggleEffect( const QString& name );
@@ -235,6 +243,7 @@ class EffectWindowImpl : public EffectWindow
         virtual bool isUserResize() const;
         virtual QRect iconGeometry() const;
         virtual QRect contentsRect() const;
+        virtual QRect decorationInnerRect() const;
         virtual QByteArray readProperty( long atom, long type, int format ) const;
         virtual void deleteProperty( long atom ) const;
 
@@ -262,6 +271,8 @@ class EffectWindowImpl : public EffectWindow
         virtual EffectWindow* findModal();
         virtual EffectWindowList mainWindows() const;
 
+        virtual bool isSkipSwitcher() const;
+
         virtual WindowQuadList buildQuads( bool force = false ) const;
 
         virtual void minimize() const;
@@ -283,7 +294,7 @@ class EffectWindowImpl : public EffectWindow
     private:
         Toplevel* toplevel;
         Scene::Window* sw; // This one is used only during paint pass.
-        QMap<int, QVariant> dataMap;
+        QHash<int, QVariant> dataMap;
 };
 
 class EffectWindowGroupImpl
