@@ -34,10 +34,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "zoom/zoom_config.h"
 
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
+#include "blur/blur_config.h"
 #include "coverswitch/coverswitch_config.h"
 #include "cube/cube_config.h"
 #include "cube/cubeslide_config.h"
 #include "flipswitch/flipswitch_config.h"
+#include "glide/glide_config.h"
 #include "invert/invert_config.h"
 #include "lookingglass/lookingglass_config.h"
 #include "mousemark/mousemark_config.h"
@@ -68,10 +70,12 @@ KWIN_EFFECT_CONFIG_MULTIPLE( builtins,
     KWIN_EFFECT_CONFIG_SINGLE( zoom, ZoomEffectConfig )
 
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    KWIN_EFFECT_CONFIG_SINGLE( blur, BlurEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( coverswitch, CoverSwitchEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( cube, CubeEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( cubeslide, CubeSlideEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( flipswitch, FlipSwitchEffectConfig )
+    KWIN_EFFECT_CONFIG_SINGLE( glide, GlideEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( invert, InvertEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( lookingglass, LookingGlassEffectConfig )
     KWIN_EFFECT_CONFIG_SINGLE( mousemark, MouseMarkEffectConfig )
