@@ -586,7 +586,7 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
             }
         if( d != highlightedDesktop ) // Highlight desktop
             {
-            if ( (me->buttons() & Qt::LeftButton) && !wasWindowMove )
+            if ( (me->buttons() & Qt::LeftButton) && !wasWindowMove && d <= effects->numberOfDesktops() )
                 {
                 EffectWindowList windows = effects->stackingOrder();
                 EffectWindowList stack;
@@ -640,7 +640,7 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
             EffectWindow* w = isDesktop ? NULL : windowAt( me->pos());
             if ( w != NULL )
                 isDesktop = w->isDesktop();
-            if( w != NULL && ( w->isMovable() || w->isMovableAcrossScreens() ))
+            if( w != NULL && ( w->isMovable() || w->isMovableAcrossScreens() || isUsingPresentWindows() ))
                 { // Prepare it for moving
                 windowMoveDiff = w->pos() - unscalePos( me->pos(), NULL );
                 windowMove = w;
@@ -747,6 +747,10 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
 
 void DesktopGridEffect::grabbedKeyboardEvent( QKeyEvent* e )
     {
+    if( timeline.value() != 1 ) // Block user input during animations
+        return;
+    if( windowMove != NULL )
+        return;
     if( e->type() == QEvent::KeyPress )
         {
         // check for global shortcuts
@@ -1250,6 +1254,11 @@ void DesktopGridEffect::setupGrid()
         case LayoutPager:
             orientation = Qt::Horizontal;
             gridSize = effects->desktopGridSize();
+            // sanity check: pager may report incorrect size in case of one desktop
+            if( numDesktops == 1 )
+                {
+                gridSize = QSize( 1, 1 );
+                }
             break;
         case LayoutAutomatic:
             y = sqrt( float( numDesktops ) ) + 0.5;
