@@ -288,9 +288,12 @@ void KCommonDecoration::resetLayout()
 
     // shown instead of the window contents in decoration previews
     if(isPreview() ) {
-        m_previewWidget = new QLabel(i18n("<center><b>%1 preview</b></center>", visibleName() ), widget());
+        m_previewWidget = new QLabel(i18n("<center><b>%1</b></center>", visibleName() ), widget());
         m_previewWidget->setAutoFillBackground(true);
         m_previewWidget->show();
+
+        // fix double deletion, see buttonDestroyed()
+        connect(m_previewWidget, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
     }
 
     addButtons(m_buttonsLeft,
@@ -310,6 +313,28 @@ void KCommonDecoration::resetLayout()
     btnHideLastWidth = 0;
 }
 
+void KCommonDecoration::objDestroyed(QObject *obj)
+{
+    // make sure button deletion is reflected in the m_button[] array.
+    // this makes sure that m_button[]-entries are not destroyed
+    // twice, first in their KCommonDecorationButton/QObject
+    // destructor (button objects are parented with the decroation
+    // widget in KCommonDecoration constructor), and then in
+    // ~KCommonDecoration.  a QPointer<KCommonDecorationButton> would
+    // have been the better approach, but changing the button array
+    // would have been ABI incompatible & would have required creation
+    // of kcommondecorationprivate instance.
+    // the same applies to m_previewWidget.
+    for (int n=0; n<NumButtons; n++) {
+        if (m_button[n] == obj) {
+            m_button[n] = 0;
+            break;
+        }
+    }
+    if(obj == m_previewWidget)
+        m_previewWidget = 0;
+}
+
 int KCommonDecoration::buttonsLeftWidth() const
 {
     return buttonContainerWidth(m_buttonsLeft);
@@ -358,6 +383,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         connect(btn, SIGNAL(pressed()), SLOT(menuButtonPressed()));
                         connect(btn, SIGNAL(released()), this, SLOT(menuButtonReleased()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[MenuButton] = btn;
                     }
                     break;
@@ -371,6 +399,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setOn( oad );
                         connect(btn, SIGNAL(clicked()), SLOT(toggleOnAllDesktops()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[OnAllDesktopsButton] = btn;
                     }
                     break;
@@ -381,6 +412,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setTipText(i18n("Help") );
                         connect(btn, SIGNAL(clicked()), SLOT(showContextHelp()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[HelpButton] = btn;
                     }
                     break;
@@ -391,6 +425,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setTipText(i18n("Minimize") );
                         connect(btn, SIGNAL(clicked()), SLOT(minimize()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[MinButton] = btn;
                     }
                     break;
@@ -405,6 +442,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setOn( max );
                         connect(btn, SIGNAL(clicked()), SLOT(slotMaximize()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[MaxButton] = btn;
                     }
                     break;
@@ -415,6 +455,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setTipText(i18n("Close") );
                         connect(btn, SIGNAL(clicked()), SLOT(closeWindow()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[CloseButton] = btn;
                     }
                     break;
@@ -428,6 +471,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setOn( above );
                         connect(btn, SIGNAL(clicked()), SLOT(slotKeepAbove()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[AboveButton] = btn;
                     }
                     break;
@@ -441,6 +487,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setOn( below );
                         connect(btn, SIGNAL(clicked()), SLOT(slotKeepBelow()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[BelowButton] = btn;
                     }
                     break;
@@ -454,6 +503,9 @@ void KCommonDecoration::addButtons(ButtonContainer &btnContainer, const QString&
                         btn->setOn( shaded );
                         connect(btn, SIGNAL(clicked()), SLOT(slotShade()));
 
+                        // fix double deletion, see objDestroyed()
+                        connect(btn, SIGNAL(destroyed(QObject*)), this, SLOT(objDestroyed(QObject*)));
+
                         m_button[ShadeButton] = btn;
                     }
                     break;
@@ -810,42 +862,13 @@ void KCommonDecoration::updateWindowShape()
     int w = widget()->width();
     int h = widget()->height();
 
-    bool tl=true,tr=true,bl=true,br=true; // is there a transparent rounded corner in top-left? etc
-
-    // no transparent rounded corners if this window corner lines up with a screen corner
-    for(int screen=0; screen < Kephal::ScreenUtils::numScreens(); ++screen)
-    {
-        QRect fullscreen(Kephal::ScreenUtils::screenGeometry(screen));
-        QRect window = geometry();
-
-        if(window.topLeft()    == fullscreen.topLeft() ) tl = false;
-        if(window.topRight()   == fullscreen.topRight() ) tr = false;
-        if(window.bottomLeft() == fullscreen.bottomLeft() ) bl = false;
-        if(window.bottomRight()== fullscreen.bottomRight() ) br = false;
-    }
-
     QRegion mask(0, 0, w, h);
 
-    // Remove top-left corner.
-    if(tl)
-    {
-        mask -= cornerShape(WC_TopLeft);
-    }
-    // Remove top-right corner.
-    if(tr)
-    {
-        mask -= cornerShape(WC_TopRight);
-    }
-        // Remove top-left corner.
-    if(bl)
-    {
-        mask -= cornerShape(WC_BottomLeft);
-    }
-    // Remove top-right corner.
-    if(br)
-    {
-        mask -= cornerShape(WC_BottomRight);
-    }
+    // Remove corners (if subclass wants to.)
+    mask -= cornerShape(WC_TopLeft);
+    mask -= cornerShape(WC_TopRight);
+    mask -= cornerShape(WC_BottomLeft);
+    mask -= cornerShape(WC_BottomRight);
 
     setMask( mask );
 }
@@ -909,9 +932,13 @@ QRect KCommonDecoration::titleRect() const
               titleEdgeBottomBottom-(r_y+titleEdgeTop) );
 }
 
+QRect KCommonDecoration::transparentRect() const
+{
+    return wrapper->transparentRect();
+}
 
 KCommonDecorationButton::KCommonDecorationButton(ButtonType type, KCommonDecoration *parent)
-    : QAbstractButton(parent->widget() ),
+    : QAbstractButton(parent?parent->widget():0 ),
         m_decoration(parent),
         m_type(type),
         m_realizeButtons(Qt::LeftButton),
@@ -1256,5 +1283,72 @@ bool KCommonDecorationUnstable::compositingActive() const
     return static_cast<const KDecorationUnstable*>( decoration() )->compositingActive();
     }
 
+// Window tabbing
+
+bool KCommonDecorationUnstable::isClientGroupActive()
+    {
+    return static_cast<KDecorationUnstable*>( decoration() )->isClientGroupActive();
+    }
+
+QList< ClientGroupItem > KCommonDecorationUnstable::clientGroupItems() const
+    {
+    return static_cast<const KDecorationUnstable*>( decoration() )->clientGroupItems();
+    }
+
+long KCommonDecorationUnstable::itemId( int index )
+    {
+    return static_cast<KDecorationUnstable*>( decoration() )->itemId( index );
+    }
+
+int KCommonDecorationUnstable::visibleClientGroupItem()
+    {
+    return static_cast<KDecorationUnstable*>( decoration() )->visibleClientGroupItem();
+    }
+
+void KCommonDecorationUnstable::setVisibleClientGroupItem( int index )
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->setVisibleClientGroupItem( index );
+    }
+
+void KCommonDecorationUnstable::moveItemInClientGroup( int index, int before )
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->moveItemInClientGroup( index, before );
+    }
+
+void KCommonDecorationUnstable::moveItemToClientGroup( long itemId, int before )
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->moveItemToClientGroup( itemId, before );
+    }
+
+void KCommonDecorationUnstable::removeFromClientGroup( int index, const QRect& newGeom )
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->removeFromClientGroup( index, newGeom );
+    }
+
+void KCommonDecorationUnstable::closeClientGroupItem( int index )
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->closeClientGroupItem( index );
+    }
+
+void KCommonDecorationUnstable::closeAllInClientGroup()
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->closeAllInClientGroup();
+    }
+
+void KCommonDecorationUnstable::displayClientMenu( int index, const QPoint& pos )
+    {
+    static_cast<KDecorationUnstable*>( decoration() )->displayClientMenu( index, pos );
+    }
+
+KDecoration::WindowOperation KCommonDecorationUnstable::buttonToWindowOperation( Qt::MouseButtons button )
+    {
+    return static_cast<KDecorationUnstable*>( decoration() )->buttonToWindowOperation( button );
+    }
+
+bool KCommonDecorationUnstable::eventFilter( QObject* o, QEvent* e )
+    {
+    return KCommonDecoration::eventFilter( o, e );
+    }
+
 // kate: space-indent on; indent-width 4; mixedindent off; indent-mode cstyle;
 
