@@ -117,6 +117,7 @@ XShmSegmentInfo SceneOpenGL::shm;
 SceneOpenGL::SceneOpenGL( Workspace* ws )
     : Scene( ws )
     , init_ok( false )
+    , selfCheckDone( false )
     {
     if( !Extensions::glxAvailable())
         {
@@ -232,6 +233,7 @@ SceneOpenGL::~SceneOpenGL()
             glXDestroyPixmap( display(), last_pixmap );
         glXDestroyContext( display(), ctxdrawable );
         }
+    EffectFrame::cleanup();
     checkGLError( "Cleanup" );
     }
 
@@ -711,7 +713,7 @@ bool SceneOpenGL::selfCheckFinish()
             || img.pixel( 1, 1 ) != QColor( Qt::black ).rgb()
             || img.pixel( 2, 1 ) != QColor( Qt::white ).rgb())
             {
-            kError( 1212 ) << "Compositing self-check failed, disabling compositing.";
+            kError( 1212 ) << "OpenGL compositing self-check failed, disabling compositing.";
             ok = false;
             break;
             }
@@ -719,7 +721,7 @@ bool SceneOpenGL::selfCheckFinish()
     if( err.error( true ))
         ok = false;
     if( ok )
-        kDebug( 1212 ) << "Compositing self-check passed.";
+        kDebug( 1212 ) << "OpenGL compositing self-check passed.";
     if( !ok && options->disableCompositingChecks )
         {
         kWarning( 1212 ) << "Compositing checks disabled, proceeding regardless of self-check failure.";
@@ -1356,8 +1358,8 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
     {
     // check if there is something to paint (e.g. don't paint if the window
     // is only opaque and only PAINT_WINDOW_TRANSLUCENT is requested)
-    bool opaque = isOpaque() && data.opacity == 1.0;
     /* HACK: It seems this causes painting glitches, disable temporarily
+    bool opaque = isOpaque() && data.opacity == 1.0;
     if(( mask & PAINT_WINDOW_OPAQUE ) ^ ( mask & PAINT_WINDOW_TRANSLUCENT ))
         { // We are only painting either opaque OR translucent windows, not both
         if( mask & PAINT_WINDOW_OPAQUE && !opaque )
@@ -1374,7 +1376,7 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
         return;
     glPushMatrix();
     // set texture filter
-    if( options->smoothScale != 0 ) // default to yes
+    if( options->glSmoothScale != 0 ) // default to yes
         {
         if( mask & PAINT_WINDOW_TRANSFORMED )
             filter = ImageFilterGood;
@@ -1390,7 +1392,7 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
         // avoid unneeded mipmap generation by only using trilinear
         // filtering when it actually makes a difference, that is with
         // minification or changed vertices
-        if( options->smoothScale == 2
+        if( options->glSmoothScale == 2
             && ( data.quads.smoothNeeded() || data.xScale < 1 || data.yScale < 1 ))
             {
             texture.setFilter( GL_LINEAR_MIPMAP_LINEAR );
@@ -1436,20 +1438,8 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
         }
     region.translate( toplevel->x(), toplevel->y() );  // Back to screen coords
 
-    texture.bind();
-    texture.enableUnnormalizedTexCoords();
-
     WindowQuadList decoration = data.quads.select( WindowQuadDecoration );
-    // paint the content
-    if ( !(mask & PAINT_DECORATION_ONLY) )
-        {
-        prepareStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
-        renderQuads( mask, region, data.quads.select( WindowQuadContents ));
-        restoreStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
-        }
 
-    texture.disableUnnormalizedTexCoords();
-    texture.unbind();
 
     // decorations
     Client *client = dynamic_cast<Client*>(toplevel);
@@ -1458,10 +1448,10 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
         {
         bool noBorder = true;
         bool updateDeco = false;
-        const QPixmap *left;
-        const QPixmap *top;
-        const QPixmap *right;
-        const QPixmap *bottom;
+        const QPixmap *left = NULL;
+        const QPixmap *top = NULL;
+        const QPixmap *right = NULL;
+        const QPixmap *bottom = NULL;
         QRect topRect, leftRect, rightRect, bottomRect;
         if( client && !client->noBorder() )
             {
@@ -1489,7 +1479,7 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
             {
             WindowQuadList topList, leftList, rightList, bottomList;
 
-            foreach( WindowQuad quad, decoration )
+            foreach( const WindowQuad& quad, decoration )
                 {
                 if( topRect.contains( QPoint( quad.originalLeft(), quad.originalTop() ) ) )
                     {
@@ -1519,6 +1509,20 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
             }
         }
 
+    texture.bind();
+    texture.enableUnnormalizedTexCoords();
+
+    // paint the content
+    if ( !(mask & PAINT_DECORATION_ONLY) )
+        {
+        prepareStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
+        renderQuads( mask, region, data.quads.select( WindowQuadContents ));
+        restoreStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
+        }
+
+    texture.disableUnnormalizedTexCoords();
+    texture.unbind();
+
     glPopMatrix();
     }
 
@@ -1549,7 +1553,7 @@ void SceneOpenGL::Window::paintDecoration( const QPixmap* decoration, TextureTyp
         // texture doesn't need updating, just bind it
         glBindTexture( decorationTexture->target(), decorationTexture->texture());
         }
-    else
+    else if( !decoration->isNull() )
         {
         bool success = decorationTexture->load( decoration->handle(), decoration->size(), decoration->depth() );
         if( !success )
@@ -1558,12 +1562,14 @@ void SceneOpenGL::Window::paintDecoration( const QPixmap* decoration, TextureTyp
             return;
             }
         }
+    else
+        return;
     if( filter == ImageFilterGood )
         {
         // avoid unneeded mipmap generation by only using trilinear
         // filtering when it actually makes a difference, that is with
         // minification or changed vertices
-        if( options->smoothScale == 2
+        if( options->glSmoothScale == 2
             && ( data.quads.smoothNeeded() || data.xScale < 1 || data.yScale < 1 ))
             {
             decorationTexture->setFilter( GL_LINEAR_MIPMAP_LINEAR );
@@ -1609,7 +1615,7 @@ void SceneOpenGL::Window::makeDecorationArrays( float** vertices, float** texcoo
     *texcoords = new float[ quads.count() * 4 * 2 ];
     float* vpos = *vertices;
     float* tpos = *texcoords;
-    foreach( WindowQuad quad, quads )
+    foreach( const WindowQuad& quad, quads )
         {
         *vpos++ = quad[ 0 ].x();
         *vpos++ = quad[ 0 ].y();
@@ -1832,6 +1838,16 @@ void SceneOpenGL::Window::prepareRenderStates( TextureType type, double opacity,
             glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, constant );
             }
         }
+    else if( !alpha && opaque )
+        {
+        float constant[] = { 1.0, 1.0, 1.0, 1.0 };
+        glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE );
+        glTexEnvi( GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_REPLACE );
+        glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE );
+        glTexEnvi( GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE );
+        glTexEnvi( GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_CONSTANT );
+        glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, constant );
+        }
     }
 
 void SceneOpenGL::Window::restoreStates( TextureType type, double opacity, double brightness, double saturation, GLShader* shader )
@@ -1887,9 +1903,9 @@ void SceneOpenGL::Window::restoreRenderStates( TextureType type, double opacity,
             glDisable( tex->target());
             glActiveTexture(GL_TEXTURE0);
             }
-        glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
-        glColor4f( 0, 0, 0, 0 );
         }
+    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
+    glColor4f( 0, 0, 0, 0 );
 
     glPopAttrib();  // ENABLE_BIT
     }
