@@ -112,6 +112,11 @@ int Unmanaged::desktop() const
     return NET::OnAllDesktops; // TODO for some window types should be the current desktop?
     }
 
+QStringList Unmanaged::activities() const
+    {
+    return QStringList();
+    }
+
 QPoint Unmanaged::clientPos() const
     {
     return QPoint( 0, 0 ); // unmanaged windows don't have decorations
