@@ -1,5 +1,5 @@
 /*  This file is part of the KDE project
-    
+
     Copyright (C) 2007 John Tapsell <tapsell@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 #include <QTextStream>
 
 //for sysconf
-#include <unistd.h>  
+#include <unistd.h>
 //for kill and setNice
 #include <sys/types.h>
 #include <signal.h>
@@ -89,7 +89,7 @@ static int ioprio_set(int which, int who, int ioprio)
 {
   return syscall(SYS_ioprio_set, which, who, ioprio);
 }
- 
+
 static int ioprio_get(int which, int who)
 {
   return syscall(SYS_ioprio_get, which, who);
@@ -114,15 +114,15 @@ namespace KSysGuard
       inline bool getNiceness(long pid, Process *process);
       inline bool getIOStatistics(const QString &dir, Process *process);
       QFile mFile;
-      char mBuffer[PROCESS_BUFFER_SIZE+1]; //used as a buffer to read data into      
+      char mBuffer[PROCESS_BUFFER_SIZE+1]; //used as a buffer to read data into
       DIR* mProcDir;
   };
 
-ProcessesLocal::Private::~Private() 
-{ 
+ProcessesLocal::Private::~Private()
+{
     closedir(mProcDir);
 }
-  
+
 ProcessesLocal::ProcessesLocal() : d(new Private())
 {
 
@@ -137,6 +137,7 @@ bool ProcessesLocal::Private::readProcStatus(const QString &dir, Process *proces
     process->uid = 0;
     process->gid = 0;
     process->tracerpid = 0;
+    process->numThreads = 0;
 
     int size;
     int found = 0; //count how many fields we found
@@ -146,25 +147,28 @@ bool ProcessesLocal::Private::readProcStatus(const QString &dir, Process *proces
 	    if((unsigned int)size > sizeof("Name:") && qstrncmp(mBuffer, "Name:", sizeof("Name:")-1) == 0) {
 	        if(process->command.isEmpty())
                 process->name = QString::fromLocal8Bit(mBuffer + sizeof("Name:")-1, size-sizeof("Name:")+1).trimmed();
-	        if(++found == 4) goto finish;
+	        if(++found == 5) goto finish;
 	    }
 	    break;
-	  case 'U': 
+	  case 'U':
 	    if((unsigned int)size > sizeof("Uid:") && qstrncmp(mBuffer, "Uid:", sizeof("Uid:")-1) == 0) {
-		sscanf(mBuffer + sizeof("Uid:") -1, "%Ld %Ld %Ld %Ld", &process->uid, &process->euid, &process->suid, &process->fsuid );
-	        if(++found == 4) goto finish;
+            sscanf(mBuffer + sizeof("Uid:") -1, "%Ld %Ld %Ld %Ld", &process->uid, &process->euid, &process->suid, &process->fsuid );
+	        if(++found == 5) goto finish;
 	    }
 	    break;
 	  case 'G':
 	    if((unsigned int)size > sizeof("Gid:") && qstrncmp(mBuffer, "Gid:", sizeof("Gid:")-1) == 0) {
-		sscanf(mBuffer + sizeof("Gid:")-1, "%Ld %Ld %Ld %Ld", &process->gid, &process->egid, &process->sgid, &process->fsgid );
-	        if(++found == 4) goto finish;
+            sscanf(mBuffer + sizeof("Gid:")-1, "%Ld %Ld %Ld %Ld", &process->gid, &process->egid, &process->sgid, &process->fsgid );
+	        if(++found == 5) goto finish;
 	    }
 	    break;
 	  case 'T':
 	    if((unsigned int)size > sizeof("TracerPid:") && qstrncmp(mBuffer, "TracerPid:", sizeof("TracerPid:")-1) == 0) {
-		process->tracerpid = atol(mBuffer + sizeof("TracerPid:")-1);
-	        if(++found == 4) goto finish;
+            process->tracerpid = atol(mBuffer + sizeof("TracerPid:")-1);
+	        if(++found == 5) goto finish;
+	    } else if((unsigned int)size > sizeof("Threads:") && qstrncmp(mBuffer, "Threads:", sizeof("Threads:")-1) == 0) {
+            process->setNumThreads(atol(mBuffer + sizeof("Threads:")-1));
+	        if(++found == 5) goto finish;
 	    }
 	    break;
 	  default:
@@ -353,7 +357,7 @@ bool ProcessesLocal::Private::readProcStatm(const QString &dir, Process *process
 
 bool ProcessesLocal::Private::readProcCmdline(const QString &dir, Process *process)
 {
-    if(!process->command.isNull()) return true; //only parse the cmdline once
+    if(!process->command.isNull()) return true; //only parse the cmdline once.  This function takes up 25% of the CPU time :-/
     mFile.setFileName(dir + "cmdline");
     if(!mFile.open(QIODevice::ReadOnly))
         return false;      /* process has terminated in the meantime */
@@ -362,13 +366,12 @@ bool ProcessesLocal::Private::readProcCmdline(const QString &dir, Process *proce
     process->command = in.readAll();
 
     //cmdline separates parameters with the NULL character
-    process->command.replace('\0', ' ');
-    process->command = process->command.trimmed();
     if(!process->command.isEmpty()) {
-        QString processname = process->command;
-        processname.remove(QRegExp("^[^ ]*/"));
-        if(processname.startsWith(process->name))
-            process->name = processname;
+        if(process->command.startsWith(process->name)) {
+            int index = process->command.indexOf(QChar('\0'));
+            process->name = process->command.left(index);
+        }
+        process->command.replace('\0', ' ');
     }
 
     mFile.close();
@@ -387,6 +390,10 @@ bool ProcessesLocal::Private::getNiceness(long pid, Process *process) {
       case (SCHED_FIFO):
 	    process->scheduler = KSysGuard::Process::Fifo;
             break;
+#ifdef SCHED_IDLE
+      case (SCHED_IDLE):
+	    process->scheduler = KSysGuard::Process::SchedulerIdle;
+#endif
 #ifdef SCHED_BATCH
       case (SCHED_BATCH):
 	    process->scheduler = KSysGuard::Process::Batch;
@@ -398,9 +405,9 @@ bool ProcessesLocal::Private::getNiceness(long pid, Process *process) {
   if(sched == SCHED_FIFO || sched == SCHED_RR) {
     struct sched_param param;
     if(sched_getparam(pid, &param) == 0)
-      process->niceLevel = param.sched_priority;
+      process->setNiceLevel(param.sched_priority);
     else
-      process->niceLevel = 0;  //Error getting scheduler parameters. 
+      process->setNiceLevel(0);  //Error getting scheduler parameters.
   }
 
 #ifdef HAVE_IONICE
@@ -412,10 +419,10 @@ bool ProcessesLocal::Private::getNiceness(long pid, Process *process) {
   }
   process->ioniceLevel = ioprio & 0xff;  /* Bottom few bits are the priority */
   process->ioPriorityClass = (KSysGuard::Process::IoPriorityClass)(ioprio >> IOPRIO_CLASS_SHIFT); /* Top few bits are the class */
+  return true;
 #else
   return false;  /* Do nothing, if we do not support this architecture */
 #endif
-  return true;
 }
 
 bool ProcessesLocal::Private::getIOStatistics(const QString &dir, Process *process)
@@ -506,7 +513,7 @@ bool ProcessesLocal::setNiceness(long pid, int priority) {
 }
 
 bool ProcessesLocal::setScheduler(long pid, int priorityClass, int priority) {
-    if(priorityClass == KSysGuard::Process::Other || priorityClass == KSysGuard::Process::Batch)
+    if(priorityClass == KSysGuard::Process::Other || priorityClass == KSysGuard::Process::Batch || priorityClass == KSysGuard::Process::SchedulerIdle)
 	    priority = 0;
     if(pid <= 0) return false; // check the parameters
     struct sched_param params;
@@ -518,6 +525,10 @@ bool ProcessesLocal::setScheduler(long pid, int priorityClass, int priority) {
 	    return (sched_setscheduler( pid, SCHED_RR, &params) == 0);
       case (KSysGuard::Process::Fifo):
 	    return (sched_setscheduler( pid, SCHED_FIFO, &params) == 0);
+#ifdef SCHED_IDLE
+      case (KSysGuard::Process::SchedulerIdle):
+	    return (sched_setscheduler( pid, SCHED_IDLE, &params) == 0);
+#endif
 #ifdef SCHED_BATCH
       case (KSysGuard::Process::Batch):
 	    return (sched_setscheduler( pid, SCHED_BATCH, &params) == 0);
