@@ -21,12 +21,14 @@
 
 #if defined __linux__
 #include "processes_linux_p.cpp"
-#elif defined __FreeBSD__ || defined __DragonFly__
+#elif defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __DragonFly__
 #include "processes_freebsd_p.cpp"
 #elif defined __OpenBSD__
 #include "processes_openbsd_p.cpp"
 #elif defined __NetBSD__
 #include "processes_netbsd_p.cpp"
+#elif defined __GNU__
+#include "processes_gnu_p.cpp"
 #else
 // Use Qt's OS detection
 #include <qglobal.h>
