@@ -28,6 +28,7 @@
 
 #include <KIcon>
 #include <KBookmarkManager>
+#include <KMimeType>
 #include <KToolInvocation>
 #include <KUrl>
 #include <KStandardDirs>
@@ -42,7 +43,6 @@ BookmarksRunner::BookmarksRunner( QObject* parent, const QVariantList &args )
     m_icon = KIcon("bookmarks");
     m_bookmarkManager = KBookmarkManager::userBookmarksManager();
     m_browser = whichBrowser();
-    m_dbCacheFile = KStandardDirs::locateLocal("cache", "") + "bookmarkrunnerfirefoxdbfile.sqlite";
     addSyntax(Plasma::RunnerSyntax(":q:", i18n("Finds web browser bookmarks matching :q:.")));
     setDefaultSyntax(Plasma::RunnerSyntax(i18nc("list of all web browser bookmarks", "bookmarks"),
                                    i18n("List all web browser bookmarks")));
@@ -55,56 +55,60 @@ BookmarksRunner::BookmarksRunner( QObject* parent, const QVariantList &args )
 
 BookmarksRunner::~BookmarksRunner()
 {
-    QFile db_CacheFile(m_dbCacheFile);
-    if (db_CacheFile.exists()) {
-        kDebug() << "Cache file was removed: " << db_CacheFile.remove();
+    if (!m_dbCacheFile.isEmpty()) {
+        QFile db_CacheFile(m_dbCacheFile);
+        if (db_CacheFile.exists()) {
+            kDebug() << "Cache file was removed: " << db_CacheFile.remove();
+        }
     }
 }
 
 void BookmarksRunner::reloadConfiguration()
 {
-    if (QSqlDatabase::isDriverAvailable("QSQLITE")) {
-        KConfigGroup grp = config();
-        /* This allows the user to specify a profile database */
-        m_dbFile = grp.readEntry<QString>("dbfile", "");
-        if (m_dbFile.isEmpty()) {//Try to get the right database file, the default profile is used
-            KConfig firefoxProfile(QDir::homePath() + "/.mozilla/firefox/profiles.ini",
-                                   KConfig::SimpleConfig);
-            QStringList profilesList = firefoxProfile.groupList();
-            profilesList = profilesList.filter(QRegExp("^Profile\\d+$"));
-            int size = profilesList.size();
+    if (m_browser == Firefox) {
+        if (QSqlDatabase::isDriverAvailable("QSQLITE")) {
+            KConfigGroup grp = config();
+            /* This allows the user to specify a profile database */
+            m_dbFile = grp.readEntry<QString>("dbfile", "");
+            if (m_dbFile.isEmpty()) {//Try to get the right database file, the default profile is used
+                KConfig firefoxProfile(QDir::homePath() + "/.mozilla/firefox/profiles.ini",
+                                       KConfig::SimpleConfig);
+                QStringList profilesList = firefoxProfile.groupList();
+                profilesList = profilesList.filter(QRegExp("^Profile\\d+$"));
+                int size = profilesList.size();
 
-            QString profilePath;
-            if (size == 1) {
-                // There is only 1 profile so we select it
-                KConfigGroup fGrp = firefoxProfile.group(profilesList.first());
-                profilePath = fGrp.readEntry("Path", "");
-            } else {
-                // There are multiple profiles, find the default one
-                foreach(const QString & profileName, profilesList) {
-                    KConfigGroup fGrp = firefoxProfile.group(profileName);
-                    if (fGrp.readEntry<int>("Default", 0)) {
-                        profilePath = fGrp.readEntry("Path", "");
-                        break;
+                QString profilePath;
+                if (size == 1) {
+                    // There is only 1 profile so we select it
+                    KConfigGroup fGrp = firefoxProfile.group(profilesList.first());
+                    profilePath = fGrp.readEntry("Path", "");
+                } else {
+                    // There are multiple profiles, find the default one
+                    foreach(const QString & profileName, profilesList) {
+                        KConfigGroup fGrp = firefoxProfile.group(profileName);
+                        if (fGrp.readEntry<int>("Default", 0)) {
+                            profilePath = fGrp.readEntry("Path", "");
+                            break;
+                        }
                     }
                 }
-            }
 
-            if (profilePath.isEmpty()) {
-              kDebug() << "No default firefox profile found";
-              m_db = QSqlDatabase();
-              return;
-            }
+                if (profilePath.isEmpty()) {
+                    kDebug() << "No default firefox profile found";
+                    m_db = QSqlDatabase();
+                    return;
+                }
 
-            profilePath.prepend(QString("%1/.mozilla/firefox/").arg(QDir::homePath()));
-            m_dbFile = profilePath + "/places.sqlite";
-            grp.writeEntry("dbfile", m_dbFile);
+                profilePath.prepend(QString("%1/.mozilla/firefox/").arg(QDir::homePath()));
+                m_dbFile = profilePath + "/places.sqlite";
+                grp.writeEntry("dbfile", m_dbFile);
+            }
+            m_db = QSqlDatabase::addDatabase("QSQLITE");
+            m_db.setHostName("localhost");
+        } else {
+            kDebug() << "SQLITE driver isn't available";
+            m_db = QSqlDatabase();
         }
-        m_db = QSqlDatabase::addDatabase("QSQLITE");
-        m_db.setHostName("localhost");
-    } else {
-        kDebug() << "SQLITE driver isn't available";
-        m_db = QSqlDatabase();
     }
 }
 
@@ -113,10 +117,37 @@ void BookmarksRunner::prep()
     m_browser = whichBrowser();
     if (m_browser == Firefox) {
         if (m_db.isValid()) {
+            if (m_dbCacheFile.isEmpty()) {
+                m_dbCacheFile = KStandardDirs::locateLocal("cache", "") + "bookmarkrunnerfirefoxdbfile.sqlite";
+            }
+
             KIO::Job *job = KIO::file_copy(m_dbFile, m_dbCacheFile, -1,
                                            KIO::HideProgressInfo | KIO::Overwrite);
             connect(job, SIGNAL(result(KJob*)), this, SLOT(dbCopied(KJob*)));
         }
+    } else if (m_browser == Opera) {
+        // open bookmarks file
+        QString operaBookmarksFilePath = QDir::homePath() + "/.opera/bookmarks.adr";
+        QFile operaBookmarksFile(operaBookmarksFilePath);
+        if (!operaBookmarksFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            kDebug() << "Could not open Operas Bookmark File " + operaBookmarksFilePath;
+            return;
+        }
+
+        // check format
+        QString firstLine = operaBookmarksFile.readLine();
+        if (firstLine.compare("Opera Hotlist version 2.0\n")) {
+            kDebug() << "Format of Opera Bookmarks File might have changed.";
+        }
+        operaBookmarksFile.readLine(); // skip options line ("Options: encoding = utf8, version=3")
+        operaBookmarksFile.readLine(); // skip empty line
+
+        // load contents
+        QString contents = operaBookmarksFile.readAll();
+        m_operaBookmarkEntries = contents.split("\n\n", QString::SkipEmptyParts);
+
+        // close file
+        operaBookmarksFile.close();
     }
 }
 
@@ -124,14 +155,18 @@ void BookmarksRunner::dbCopied(KJob *)
 {
     m_db.setDatabaseName(m_dbCacheFile);
     m_dbOK = m_db.open();
-    //kDebug() << "Database was opened: " << m_dbOK;
+    kDebug() << "Database was opened: " << m_dbOK;
 }
 
 void BookmarksRunner::down()
 {
-    if (m_db.isOpen()) {
-        m_db.close();
-        m_dbOK = false;
+    if (m_browser == Firefox) {
+        if (m_db.isOpen()) {
+            m_db.close();
+            m_dbOK = false;
+        }
+    } else if (m_browser == Opera) {
+        m_operaBookmarkEntries.clear();
     }
 }
 
@@ -141,34 +176,115 @@ void BookmarksRunner::match(Plasma::RunnerContext &context)
     if ((term.length() < 3) && (!context.singleRunnerQueryMode())) {
         return;
     }
+
     bool allBookmarks = term.compare(i18nc("list of all konqueror bookmarks", "bookmarks"),
                                      Qt::CaseInsensitive) == 0;
-    if (m_browser == Konqueror) {
-        matchKonquerorBookmarks(context, allBookmarks, term);
-    } else if (m_browser == Firefox) {
-        matchFirefoxBookmarks(context, allBookmarks, term);
+    switch (m_browser) {
+        case Firefox:
+            matchFirefoxBookmarks(context, allBookmarks, term);
+            break;
+        case Opera:
+            matchOperaBookmarks(context, allBookmarks, term);
+            break;
+        case Default:
+            matchKonquerorBookmarks(context, allBookmarks, term);
+            break;
     }
 }
 
 KIcon BookmarksRunner::favicon(const KUrl &url)
 {
     // query the favicons module
-    QDBusInterface favicon("org.kde.kded", "/modules/favicons", "org.kde.FavIcon");
-    QDBusReply<QString> reply = favicon.call("iconForUrl", url.url());
+    const QString iconFile = KMimeType::favIconForUrl(url);
 
-    if (!reply.isValid()) {
-        return KIcon();
+    if (iconFile.isEmpty()) {
+        return m_icon;
     }
 
-    // locate the favicon
-    const QString iconFile = KGlobal::dirs()->findResource("cache",reply.value()+".png");
-    if(iconFile.isNull()) {
-        return KIcon();
+    return KIcon(iconFile);
+}
+
+void BookmarksRunner::matchOperaBookmarks(Plasma::RunnerContext& context, bool allBookmarks,
+                                                        const QString& term)
+{
+    QList<Plasma::QueryMatch> matches;
+
+    QLatin1String nameStart("\tNAME=");
+    QLatin1String urlStart("\tURL=");
+    QLatin1String descriptionStart("\tDESCRIPTION=");
+
+    // search
+    foreach (const QString & entry, m_operaBookmarkEntries) {
+        if (!context.isValid()) {
+            return;
+        }
+
+        QStringList entryLines = entry.split("\n");
+        if (!entryLines.first().startsWith(QString("#URL"))) {
+            continue; // skip folder entries
+        }
+        entryLines.pop_front();
+
+        QString name;
+        QString url;
+        QString description;
+
+        foreach (const QString & line, entryLines) {
+            if (line.startsWith(nameStart)) {
+                name = line.mid( QString(nameStart).length() ).simplified();
+            } else if (line.startsWith(urlStart)) {
+                url = line.mid( QString(urlStart).length() ).simplified();
+            } else if (line.startsWith(descriptionStart)) {
+                description = line.mid(QString(descriptionStart).length())
+                              .simplified();
+            }
+        }
+
+        if (url.simplified().isEmpty())
+            continue; // skip useless entries
+
+        Plasma::QueryMatch::Type type = Plasma::QueryMatch::NoMatch;
+        qreal relevance = 0;
+
+        if (name.compare(term, Qt::CaseInsensitive) == 0
+             || description.compare(term, Qt::CaseInsensitive) == 0) {
+            type = Plasma::QueryMatch::ExactMatch;
+            relevance = 1.0;
+        } else if (name.contains(term, Qt::CaseInsensitive)) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.45;
+        } else if (description.contains(term, Qt::CaseInsensitive)) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.3;
+        } else if (url.contains(term, Qt::CaseInsensitive)) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.2;
+        } else if (allBookmarks) {
+            type = Plasma::QueryMatch::PossibleMatch;
+            relevance = 0.18;
+        }
+
+        if (type != Plasma::QueryMatch::NoMatch) {
+            bool isNameEmpty = name.isEmpty();
+            bool isDescriptionEmpty = description.isEmpty();
+
+            Plasma::QueryMatch match(this);
+            match.setType(type);
+            match.setRelevance(relevance);
+            match.setIcon(m_icon);
+
+            // Try to set the following as text in this order: name, description, url
+            match.setText( isNameEmpty
+                           ?
+                           (!isDescriptionEmpty ? description : url)
+                           :
+                           name );
+
+            match.setData(url);
+            matches << match;
+        }
     }
-
-    KIcon icon = KIcon(iconFile);
-
-    return icon;
+    context.addMatches(term, matches);
 }
 
 void BookmarksRunner::matchKonquerorBookmarks(Plasma::RunnerContext& context, bool allBookmarks,
@@ -225,20 +341,17 @@ void BookmarksRunner::matchKonquerorBookmarks(Plasma::RunnerContext& context, bo
 
         if (type != Plasma::QueryMatch::NoMatch) {
             //kDebug() << "Found bookmark: " << bookmark.text() << " (" << bookmark.url().prettyUrl() << ")";
-            // getting the favicon is too slow and can easily lead to starving the thread pool out
-            /*
-            QIcon icon = getFavicon(bookmark.url());
-            if (icon.isNull()) {
-                match->setIcon(m_icon);
-            }
-            else {
-                match->setIcon(icon);
-            }
-            */
             Plasma::QueryMatch match(this);
+
+            QIcon icon = favicon(bookmark.url());
+            if (icon.isNull()) {
+                match.setIcon(m_icon);
+            } else {
+                match.setIcon(icon);
+            }
+
             match.setType(type);
             match.setRelevance(relevance);
-            match.setIcon(m_icon);
             match.setText(bookmark.text());
             match.setData(bookmark.url().url());
             matches << match;
@@ -314,11 +427,11 @@ BookmarksRunner::Browser BookmarksRunner::whichBrowser()
 {
     KConfigGroup config(KSharedConfig::openConfig("kdeglobals"), QLatin1String("General") );
     QString exec = config.readPathEntry( QLatin1String("BrowserApplication"), QString("") );
-    kDebug() << exec;
+    //kDebug() << exec;
     if (exec.contains("firefox", Qt::CaseInsensitive)) {
         return Firefox;
-    } else if (exec.contains("konqueror", Qt::CaseInsensitive)) {
-        return Konqueror;
+    } else if (exec.contains("opera", Qt::CaseInsensitive)) {
+        return Opera;
     } else {
         return Default;
     }
@@ -332,4 +445,16 @@ void BookmarksRunner::run(const Plasma::RunnerContext &context, const Plasma::Qu
     KToolInvocation::invokeBrowser(url.url());
 }
 
+QMimeData * BookmarksRunner::mimeDataForMatch(const Plasma::QueryMatch * match)
+{
+    QMimeData * result = new QMimeData();
+    QList<QUrl> urls;
+    urls << QUrl(match->data().toString());
+    result->setUrls(urls);
+
+    result->setText(match->data().toString());
+
+    return result;
+}
+
 #include "bookmarksrunner.moc"
